"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ExpressAuthorizationProvider = void 0;
class ExpressAuthorizationProvider {
    constructor(options) {
        this.functionList = {};
        this.debug = (options === null || options === void 0 ? void 0 : options.debug) ? options.debug : false;
        this.failureHandler = (options === null || options === void 0 ? void 0 : options.failureHandler)
            ? options.failureHandler
            : this.defaultFailureHandler;
        this.dontUseFailureHandler = (options === null || options === void 0 ? void 0 : options.dontUseFailureHandler)
            ? options.dontUseFailureHandler
            : false;
    }
    defaultFailureHandler(_req, res, action) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.debug) {
                console.debug("[Express Authorization Provider] - ERROR HANDLER ACTION: " + action);
            }
            res.status(403).json(`You do not have permission to: [${action}]`);
        });
    }
    use(action, functionHandler) {
        if (typeof functionHandler === "function") {
            if (this.debug) {
                console.debug(`[Express Authorization Provider] - Registering [${action}]`);
            }
            this.functionList[action] = functionHandler;
        }
        else {
            throw new Error("[Express Authorization Provider] - You can only register handlers that are functions! Please see the docs for more info!");
        }
    }
    can(action) {
        return (req, res, next) => __awaiter(this, void 0, void 0, function* () {
            const handler = this.functionList[action];
            if (handler) {
                if (this.debug) {
                    console.debug(`[Express Authorization Provider] - Handler Called: [${action}]`);
                }
                try {
                    handler(req, res, next);
                    next();
                }
                catch (error) {
                    if (this.dontUseFailureHandler) {
                        next(error);
                    }
                    else {
                        yield this.failureHandler(req, res, action);
                    }
                }
            }
            else {
                next(new Error(`[Express Authorization Provider] - The specified provider action was not registered: [${action}]`));
            }
        });
    }
    is(noun) {
        return this.can(noun);
    }
}
exports.ExpressAuthorizationProvider = ExpressAuthorizationProvider;
//# sourceMappingURL=express-authorization-provider.js.map